VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSpecialEffects"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True


'These arrays are the "lookup tables" for the effects (the first
'dimension is the actual table index). They contain the pre-calculated
'results of the effects in every possible color combination.
Dim LookupTable() As Byte

'If it's been initialized or not
Dim AlreadyInitialized As Boolean


'This sub initializes the lookup tables. I know it seems huge because
'of the big number of effects, but if you go trough it step by step
'it won't be too hard to understand!
Public Sub InitEffects()
    Dim SrcColor As Long, DestColor As Long
    
    
    'Resize them!
    ReDim LookupTable(0 To 17, 255, 255)
    
    
    'Fill the "Add" (0) lookup table
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            'Add the values of both colors (remember this lookup table is generic,
            'so this applies to all of the R, G and B values)
            TempColor = DestColor + SrcColor
            
            'The maximum is 255!
            If TempColor > 255 Then TempColor = 255
            
            'Save the result into the lookup table
            LookupTable(0, DestColor, SrcColor) = TempColor
        Next SrcColor
    Next DestColor
    
    
    'Fill the "Subtract (with no color correction)" (1) lookup table
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            'Subtract the values of both colors (remember this lookup table is
            'generic, so this applies to all of the R, G and B values)
            TempColor = DestColor - SrcColor
            
            'The minimum is 0!
            If TempColor < 0 Then TempColor = 0
            
            'Save the result into the lookup table
            LookupTable(1, DestColor, SrcColor) = TempColor
        Next SrcColor
    Next DestColor
    
    
    'Fill the "Subtract (corrected)" (2) lookup table
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            'Subtract the values of both colors, but by inverting the source
            'color, so the final color is not inverted
            TempColor = DestColor - (255 - SrcColor)
            
            'The minimum is 0!
            If TempColor < 0 Then TempColor = 0
            
            'Save the result into the lookup table
            LookupTable(2, DestColor, SrcColor) = TempColor
        Next SrcColor
    Next DestColor
    
    
    'Fill the "Better Add Effect (Alpha Map to White)" (3) lookup table
    Dim FinalSrcColor As Byte
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            'This is a trick that uses the source image as an alpha map,
            'and White as the source color. It's an enhanced Add effect,
            'because it makes the destination brighter but not too much!
            'The algorythm is the same as for translucency, but changed
            'a bit to make it faster (since the color is constant!)
            
            FinalSrcColor = 255 - SrcColor 'Invert the source
            TempColor = (FinalSrcColor * (DestColor + 1)) / 256 + 255 - FinalSrcColor
            
            'Save the result into the lookup table
            LookupTable(3, DestColor, SrcColor) = TempColor
        Next SrcColor
    Next DestColor
    
    
    'Fill the "Better Subtract Effect (Alpha Map to Black)" (4) lookup table
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            'This is a trick that uses the source image as an alpha map,
            'and Black as the source color. It's an enhanced Add effect,
            'because it makes the destination brighter but not too much!
            'The algorythm is the same as for translucency, but changed
            'a bit to make it faster (since the color is constant!)
            
            TempColor = (SrcColor * (DestColor + 256)) / 256 - SrcColor
            
            'Save the result into the lookup table
            LookupTable(4, DestColor, SrcColor) = TempColor
        Next SrcColor
    Next DestColor
    
    
    'Fill the "Translucency" (5 to 12) lookup tables
    Dim FinalAlpha As Long
    
    'Loop trough all translucency levels (8 - I think that's enough :p )
    Dim i As Long
    For i = 0 To 7
        'Loop trough all possible pixel combinations...
        For DestColor = 0 To 255
            For SrcColor = 0 To 255
                'This little algorythm was made by David Goodlad (author of
                'vbDABL). It allowed for 256 levels of translucency, but I
                'modified it to 10 (only 8 are available because the other 2
                'are completely opaque and completely transparent, so they
                'aren't needed :p )
                'If you need more, it's just as easy: just ReDim the array to
                'a bigger number, and increase the "11" to another value.
                'Thanks, David! ;)
                
                If SrcColor > 0 Then
                    FinalAlpha = i + 2
                    TempColor = (FinalAlpha * ((DestColor + 11) - SrcColor)) / 11 + SrcColor - FinalAlpha
                Else
                    TempColor = DestColor 'Use black as totally transparent
                End If
                
                'Save the result into the lookup table
                LookupTable(i + 5, DestColor, SrcColor) = TempColor
            Next SrcColor
        Next DestColor
    Next i
    
    
    
    'Fill the "Weird Effects" (13 to 17) lookup tables
    'These functions were made by trial-and-error ;)
    
    'Loop trough all possible pixel combinations...
    For DestColor = 0 To 255
        For SrcColor = 0 To 255
            If SrcColor > 0 Then
                '--DestColor * Atn(DestColor / SrcColor)
                TempColor = DestColor * Atn(DestColor / SrcColor)
                
                'Must be between 0 and 255!
                If TempColor > 255 Then TempColor = 255
                If TempColor < 0 Then TempColor = 0
                
                'Save the result into the lookup table
                LookupTable(13, DestColor, SrcColor) = TempColor
                
                
                '--DestColor * Sqr(DestColor / SrcColor)
                TempColor = DestColor * Sqr(DestColor / SrcColor)
                
                'Must be between 0 and 255!
                If TempColor > 255 Then TempColor = 255
                If TempColor < 0 Then TempColor = 0
                
                'Save the result into the lookup table
                LookupTable(14, DestColor, SrcColor) = TempColor
                
                
                '--SrcColor * Sin(DestColor / SrcColor)
                TempColor = SrcColor * Sin(DestColor / SrcColor)
                
                'Must be between 0 and 255!
                If TempColor > 255 Then TempColor = 255
                If TempColor < 0 Then TempColor = 0
                
                'Save the result into the lookup table
                LookupTable(15, DestColor, SrcColor) = TempColor
                
                
                '--SrcColor * Cos(DestColor / SrcColor)
                TempColor = SrcColor * Cos(DestColor / SrcColor)
                
                'Must be between 0 and 255!
                If TempColor > 255 Then TempColor = 255
                If TempColor < 0 Then TempColor = 0
                
                'Save the result into the lookup table
                LookupTable(16, DestColor, SrcColor) = TempColor
                
                
                '--Sqr(DestColor) * Sqr(SrcColor)
                TempColor = Sqr(DestColor) * Sqr(SrcColor)
                
                'Must be between 0 and 255!
                If TempColor > 255 Then TempColor = 255
                If TempColor < 0 Then TempColor = 0
                
                'Save the result into the lookup table
                LookupTable(17, DestColor, SrcColor) = TempColor
            Else
                For i = 13 To 17
                    LookupTable(i, DestColor, SrcColor) = DestColor 'Use black as totally transparent
                Next i
            End If
        Next SrcColor
    Next DestColor
    
    AlreadyInitialized = True
End Sub

'This function will loop trough the elements of two arrays and apply the effects
'to them.
Public Sub SpecialBltArray(SrcArray() As Byte, DestArray() As Byte, ByVal dX As Long, ByVal dY As Long, SrcX As Long, SrcY As Long, SrcWid As Long, SrcHgt As Long, LookupTableIndex As Long)
    'On Error Resume Next
    
    Dim cX As Long, cY As Long
    Dim AbsX As Long, AbsY As Long
    
    If Not AlreadyInitialized Then InitEffects
    
    'Loop trough all pixels...
    For cX = 0 To SrcWid
        For cY = 0 To SrcHgt
            'See the resulting value of crossing the effect's R/G/B value with the
            'destination's RGB values, in the lookup table.
            AbsX = dX + cX
            AbsY = dY + cY
            DestArray(AbsX, AbsY) = LookupTable(LookupTableIndex, DestArray(AbsX, AbsY), SrcArray(SrcX + cX, SrcY + cY))
        Next cY
    Next cX
End Sub
