Attribute VB_Name = "DirectX"

Option Explicit


Public Type KSurface
    SURF As DirectDrawSurface7
    DESC As DDSURFACEDESC2
    Width As Long
    Height As Long
End Type

Public Type PointAPI
    X As Long
    Y As Long
End Type


Global bInit As Boolean                  'Used by the program to see if everythings been started.
Global DirectX As New DirectX7           'Master Object, Everything is created from this.
Global DDraw As DirectDraw7              'The DirectDraw Object, this is created from the DirectX Object

'The keyboard
Global DirectInput As DirectInput
Global Keyboard As DirectInputDevice
Global KeyboardState As DIKEYBOARDSTATE

Global Primary As KSurface               'What's on the screen
Global Backbuffer As KSurface            'The backbuffer

Global oClipper As DirectDrawClipper     'The clipper object

Global ErrorMsg As String                'A little error message (like "File not found", etc)

Global ScreenX As Long
Global ScreenY As Long

'Closes DX
Sub CloseDX()
    Call DDraw.RestoreDisplayMode
    Call DDraw.SetCooperativeLevel(frmGame.hWnd, DDSCL_NORMAL)
    
    Keyboard.Unacquire
    Set Keyboard = Nothing
    Set DirectInput = Nothing
    
    Unload frmGame
    
    'Show an error message
    If ErrorMsg <> "" Then MsgBox ErrorMsg, vbOKOnly, "Error"
    
    'End the program
    End
End Sub

'Clears a surface (eg.: the backbuffer)
Sub ClearSurface(Dest As KSurface, Optional Color As Long = 0)
    Dim FillRect As RECT
    With FillRect
        .Top = 0: .Left = 0
        .Right = 0: .Bottom = 0
    End With
    Dest.SURF.BltColorFill FillRect, Color
End Sub

'Initialize DX
Function InitializeDX(ResX As Long, ResY As Long)
    On Error GoTo ErrorHandler
    
    'Create the keyboard object
    Set DirectInput = DirectX.DirectInputCreate()
    Set Keyboard = DirectInput.CreateDevice("GUID_SysKeyboard")
    Keyboard.SetCommonDataFormat DIFORMAT_KEYBOARD
    Keyboard.SetCooperativeLevel frmGame.hWnd, DISCL_NONEXCLUSIVE Or DISCL_BACKGROUND
    
    'Create the directdraw object
    Set DDraw = DirectX.DirectDrawCreate("")
    
    'Set the cooperative level to exclusive and fullscreen
    Call DDraw.SetCooperativeLevel(frmGame.picDisplay.hWnd, DDSCL_NORMAL)
    
    'Create the primary surface with one backbuffer
    Primary.DESC.lFlags = DDSD_CAPS
    Primary.DESC.ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE
    Set Primary.SURF = DDraw.CreateSurface(Primary.DESC)
    
    'Get a backbuffer from the primary surface
    Backbuffer.DESC.lFlags = DDSD_CAPS Or DDSD_WIDTH Or DDSD_HEIGHT
    Backbuffer.DESC.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
    Backbuffer.DESC.lWidth = ResX
    Backbuffer.DESC.lHeight = ResY
    Set Backbuffer.SURF = DDraw.CreateSurface(Backbuffer.DESC)
    
    'Set the clipper
    Set oClipper = DDraw.CreateClipper(0)
    oClipper.SetHWnd frmGame.picDisplay.hWnd
    Primary.SURF.SetClipper oClipper
    
    bInit = True
    
    Exit Function
ErrorHandler:
    'If anything goes wrong, close the program
    CloseDX
End Function

'Flips the primary surface
Sub Flip()
    Dim DestRect As RECT
    Dim TempRect As RECT
    
    DirectX.GetWindowRect frmGame.picDisplay.hWnd, DestRect
    Primary.SURF.Blt DestRect, Backbuffer.SURF, TempRect, DDBLT_WAIT
End Sub

'Just like regular BitBlt, but with surfaces
Sub Blt(Source As KSurface, Dest As KSurface, ByVal dX As Long, ByVal dY As Long, SrcX As Long, SrcY As Long, SrcWid As Long, SrcHgt As Long, Optional Transparent As Boolean = False)
    Dim SrcRect As RECT
    
    With SrcRect
    
    'Left
    If dX <= 0 Then
        .Left = -dX + SrcX
        dX = 0
    Else
        .Left = SrcX
    End If
    
    'Top
    If dY <= 0 Then
        .Top = -dY + SrcY
        dY = 0
    Else
        .Top = SrcY
    End If
    
    'Right
    If dX + SrcWid > ScreenX Then
        .Right = SrcX + SrcWid - ((dX + SrcWid) - ScreenX)
    Else
        .Right = SrcX + SrcWid
    End If
    
    'Bottom
    If dY + SrcHgt > ScreenY Then
        .Bottom = SrcY + SrcHgt - ((dY + SrcHgt) - ScreenY)
    Else
        .Bottom = SrcY + SrcHgt
    End If
    
    End With
    
    'Draw it!
    If Transparent Then 'If it needs to be transparent
        Dest.SURF.BltFast dX, dY, Source.SURF, SrcRect, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY
    Else
        Dest.SURF.BltFast dX, dY, Source.SURF, SrcRect, DDBLTFAST_WAIT
    End If
End Sub

'Loads a surface from a file
Sub LoadSurface(FileName As String, Dest As KSurface, PixelWidth As Long, PixelHeight As Long, Optional SpecialEffect As Boolean = False)
    'First, check if the file exists!
    If FileName = "" Or Dir(FileName) = "" Then
        'File not found!
        ErrorMsg = "File not found: " & FileName
        CloseDX
    End If
    
    'Set the surface's width and height
    Dest.Width = PixelWidth
    Dest.Height = PixelHeight
    
    'This piece of code will load the surface:
    
    'Set the flags
    Dest.DESC.lFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
    
    If Not SpecialEffect Then
        'Load it as a regular surface
        Dest.DESC.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
    Else
        'System memory is faster when it comes to special effects :)
        Dest.DESC.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN Or DDSCAPS_SYSTEMMEMORY
    End If
    
    'Width/height of the image...
    Dest.DESC.lWidth = PixelWidth
    Dest.DESC.lHeight = PixelHeight
    
    'Create it...
    Set Dest.SURF = DDraw.CreateSurfaceFromFile(FileName, Dest.DESC)
    
    'Set the color key to black
    Dim Key As DDCOLORKEY
    Key.low = 0
    Key.high = Key.low
    Dest.SURF.SetColorKey DDCKEY_SRCBLT, Key
End Sub
